@@ -16,6 +16,9 @@
     the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
     Boston, MA 02110-1301, USA.
 */
+
+#include "kio_man.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
@@ -40,7 +43,6 @@
 #include <klocale.h>
 #include <kmimetype.h>
 
-#include "kio_man.h"
 #include "kio_man.moc"
 #include "man2html.h"
 #include <assert.h>
@@ -83,8 +85,8 @@ bool parseUrl(const QString& _url, QString &title, QString &section)
     section.clear();
 
     QString url = _url;
-    if (url.at(0) == '/') {
-        if (KStandardDirs::exists(url)) {
+    if (url.isEmpty() || url.at(0) == '/') {
+        if (url.isEmpty() || KStandardDirs::exists(url)) {
             // man:/usr/share/man/man1/ls.1.gz is a valid file
             title = url;
             return true;
@@ -107,7 +109,14 @@ bool parseUrl(const QString& _url, QString &title, QString &section)
     title = title.left(pos);
 
     section = url.mid(pos+1);
-    section = section.left(section.length()-1);
+
+    pos = section.indexOf(')');
+    if (pos >= 0) {
+	if (pos < section.length() - 2 && title.isEmpty()) {
+		title = section.mid(pos + 2);
+	}
+        section = section.left(pos);
+    }
 
     // man:ls(2) -> title="ls", section="2"
 
@@ -126,7 +135,10 @@ MANProtocol::MANProtocol(const QByteArray &pool_socket, const QByteArray &app_so
     m_cssPath=strPath.toLocal8Bit(); // ### TODO encode for CSS
     section_names << "1" << "2" << "3" << "3n" << "3p" << "4" << "5" << "6" << "7"
                   << "8" << "9" << "l" << "n";
-    m_manCSSFile = KStandardDirs::locate( "data", "kio_man/kio_man.css" );
+
+    QString cssPath(KStandardDirs::locate( "data", "kio_man/kio_man.css" ));
+    KUrl cssUrl(KUrl::fromPath(cssPath));
+    m_manCSSFile = cssUrl.url().toUtf8();
 }
 
 MANProtocol *MANProtocol::self() { return _self; }
@@ -481,10 +493,10 @@ void MANProtocol::get(const KUrl& url )
     {
        outputError(i18n("No man page matching to %1 found.<br /><br />"
            "Check that you have not mistyped the name of the page that you want.<br />"
-           "Be careful that you must take care about upper case and lower case characters!<br />"
-           "If everything looks correct, then perhaps you need to set a better search path "
-           "for man pages, be it by the environment variable MANPATH or a matching file "
-           "in the directory /etc .", text2html(title)));
+           "Check that you have typed the name using the correct upper and lower case characters.<br />"
+           "If everything looks correct, then you may need to improve the search path "
+           "for man pages; either using the environment variable MANPATH or using a matching file "
+           "in the /etc directory.", text2html(title)));
        pageFound=false;
     }
     else if (foundPages.count()>1)
@@ -504,6 +516,7 @@ void MANProtocol::get(const KUrl& url )
     if (pageFound)
     {
        setResourcePath(m_htmlPath,m_cssPath);
+       setCssFile(m_manCSSFile);
        m_outputBuffer.open(QIODevice::WriteOnly);
        const QByteArray filename=QFile::encodeName(foundPages[0]);
        char *buf = readManPage(filename);
@@ -616,9 +629,9 @@ void MANProtocol::outputError(const QString& errmsg)
     os << "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">" << endl;
     os << "<title>" << i18n("Man output") << "</title>\n" << endl;
     if ( !m_manCSSFile.isEmpty() )
-        os << "<link href=\"file:///" << m_manCSSFile << "\" type=\"text/css\" rel=\"stylesheet\">" << endl;
+        os << "<link href=\"" << m_manCSSFile << "\" type=\"text/css\" rel=\"stylesheet\">" << endl;
     os << "</head>" << endl;
-    os << i18n("<body><h1>KDE Man Viewer Error</h1>") << errmsg << "</body>" << endl;
+    os << "<body>" << i18n("<h1>KDE Man Viewer Error</h1>") << errmsg << "</body>" << endl;
     os << "</html>" << endl;
 
     data(array);
@@ -634,7 +647,7 @@ void MANProtocol::outputMatchingPages(const QStringList &matchingPages)
     os << "<html>\n<head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">"<<endl;
     os << "<title>" << i18n("Man output") <<"</title>" << endl;
     if ( !m_manCSSFile.isEmpty() )
-        os << "<link href=\"file:///" << m_manCSSFile << "\" type=\"text/css\" rel=\"stylesheet\">" << endl;
+        os << "<link href=\"" << m_manCSSFile << "\" type=\"text/css\" rel=\"stylesheet\">" << endl;
     os << "</head>" <<endl;
     os << "<body><h1>" << i18n("There is more than one matching man page.");
     os << "</h1>\n<ul>\n";
@@ -785,7 +798,7 @@ void MANProtocol::showMainIndex()
     os << "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">" << endl;
     os << "<title>" << i18n("UNIX Manual Index") << "</title>" << endl;
     if (!m_manCSSFile.isEmpty())
-        os << "<link href=\"file:///" << m_manCSSFile << "\" type=\"text/css\" rel=\"stylesheet\">" << endl;
+        os << "<link href=\"" << m_manCSSFile << "\" type=\"text/css\" rel=\"stylesheet\">" << endl;
     os << "</head>" << endl;
     os << "<body><h1>" << i18n("UNIX Manual Index") << "</h1>" << endl;
 
@@ -1215,7 +1228,7 @@ void MANProtocol::showIndex(const QString& section)
     os << "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">" << endl;
     os << "<title>" << i18n("UNIX Manual Index") << "</title>" << endl;
     if ( !m_manCSSFile.isEmpty() )
-        os << "<link href=\"file:///" << m_manCSSFile << "\" type=\"text/css\" rel=\"stylesheet\">" << endl;
+        os << "<link href=\"" << m_manCSSFile << "\" type=\"text/css\" rel=\"stylesheet\">" << endl;
     os << "</head>" << endl;
     os << "<body><div class=\"secidxmain\">" << endl;
     os << "<h1>" << i18n( "Index for Section %1: %2", section, sectionName(section)) << "</h1>" << endl;
@@ -1485,7 +1498,7 @@ void MANProtocol::showIndex(const QString& section)
 
 void MANProtocol::listDir(const KUrl &url)
 {
-    kDebug( 7107 ) << "ENTER listDir: " << url.prettyUrl();
+    kDebug( 7107 ) << url;
 
     QString title;
     QString section;
@@ -1500,14 +1513,27 @@ void MANProtocol::listDir(const KUrl &url)
     // But we cannot list man:ls as a directory, this makes no sense (#154173)
 
     if (!title.isEmpty() && title != "/") {
-        error(KIO::ERR_IS_FILE, url.url());
+	error(KIO::ERR_IS_FILE, url.url());
         return;
     }
 
-    QStringList list = findPages( section, QString(), false );
-
     UDSEntryList uds_entry_list;
 
+    if (section.isEmpty()) {
+        for (QStringList::ConstIterator it = section_names.begin(); it != section_names.end(); ++it) {
+            UDSEntry     uds_entry;
+
+            QString name = "man:/(" + *it + ")";
+            uds_entry.insert( KIO::UDSEntry::UDS_NAME, sectionName( *it ) );
+            uds_entry.insert( KIO::UDSEntry::UDS_URL, name );
+            uds_entry.insert( KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR );
+
+            uds_entry_list.append( uds_entry );
+        }
+    }
+
+    QStringList list = findPages( section, QString(), false );
+
     QStringList::Iterator it = list.begin();
     const QStringList::Iterator end = list.end();
 
