@@ -20,6 +20,7 @@
 */
 
 #include "backend.h"
+//#include <phonon/experimental/backendinterface.h>
 #include "mediaobject.h"
 #include "effect.h"
 #include "events.h"
@@ -28,7 +29,7 @@
 #include "nullsink.h"
 #include "visualization.h"
 #include "volumefadereffect.h"
-#include "videodataoutput.h"
+//#include "videodataoutput.h"
 #include "videowidget.h"
 #include "wirecall.h"
 #include "xinethread.h"
@@ -44,9 +45,6 @@
 #include <QtCore/QVariant>
 #include <QtGui/QApplication>
 
-#include <phonon/audiodevice.h>
-#include <phonon/audiodeviceenumerator.h>
-
 extern "C" {
 #include <xine/xine_plugin.h>
 #include "shareddata.h"
@@ -109,8 +107,8 @@ QObject *Backend::createObject(BackendInterface::Class c, QObject *parent, const
         return new AudioDataOutput(parent);
     case VisualizationClass:
         return new Visualization(parent);
-    case VideoDataOutputClass:
-        return new VideoDataOutput(parent);
+    //case Phonon::Experimental::BackendInterface::VideoDataOutputClass:
+        //return new VideoDataOutput(parent);
     case EffectClass:
         {
             Q_ASSERT(args.size() == 1);
@@ -169,7 +167,7 @@ QStringList Backend::availableMimeTypes() const
         QString mimeTypes(mimeTypes_c);
         free(mimeTypes_c);
         QStringList lstMimeTypes = mimeTypes.split(";", QString::SkipEmptyParts);
-        foreach (QString mimeType, lstMimeTypes)
+        foreach (const QString &mimeType, lstMimeTypes)
             m_supportedMimeTypes << mimeType.left(mimeType.indexOf(':')).trimmed();
         if (m_supportedMimeTypes.contains("application/ogg"))
             m_supportedMimeTypes << QLatin1String("audio/x-vorbis+ogg") << QLatin1String("application/ogg");
@@ -185,15 +183,9 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
     {
     case Phonon::AudioOutputDeviceType:
         return XineEngine::audioOutputIndexes();
-/*
     case Phonon::AudioCaptureDeviceType:
-        {
-            QList<AudioDevice> devlist = AudioDeviceEnumerator::availableCaptureDevices();
-            foreach (AudioDevice dev, devlist) {
-                list << dev.index();
-            }
-        }
         break;
+/*
     case Phonon::VideoOutputDeviceType:
         {
             const char *const *outputPlugins = xine_list_video_output_plugins(XineEngine::xine());
@@ -224,6 +216,15 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
             } */
             break;
         }
+    case Phonon::AudioChannelType:
+    case Phonon::SubtitleType:
+        {
+            ObjectDescriptionHash hash = XineEngine::objectDescriptions();
+            ObjectDescriptionHash::iterator it = hash.find(type);
+            if( it != hash.end() )
+                list = it.value().keys();
+        }
+        break;
     }
     return list;
 }
@@ -236,39 +237,9 @@ QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescripti
     case Phonon::AudioOutputDeviceType:
         ret = XineEngine::audioOutputProperties(index);
         break;
-        /*
     case Phonon::AudioCaptureDeviceType:
-        {
-            QList<AudioDevice> devlist = AudioDeviceEnumerator::availableCaptureDevices();
-            foreach (AudioDevice dev, devlist) {
-                if (dev.index() == index) {
-                    ret.insert("name", dev.cardName());
-                    switch (dev.driver()) {
-                    case Solid::AudioInterface::Alsa:
-                        ret.insert("description", i18n("ALSA Capture Device"));
-                        break;
-                    case Solid::AudioInterface::OpenSoundSystem:
-                        ret.insert("description", i18n("OSS Capture Device"));
-                        break;
-                    case Solid::AudioInterface::UnknownAudioDriver:
-                        break;
-                    }
-                    ret.insert("icon", dev.iconName());
-                    ret.insert("available", dev.isAvailable());
-                    break;
-                }
-            }
-        }
-        switch (index) {
-        case 20000:
-            ret.insert("name", QLatin1String("Soundcard"));
-            break;
-        case 20001:
-            ret.insert("name", QLatin1String("DV"));
-            break;
-        }
-        //kDebug(610) << ret["name"];
         break;
+        /*
     case Phonon::VideoOutputDeviceType:
         {
             const char *const *outputPlugins = xine_list_video_output_plugins(XineEngine::xine());
@@ -324,6 +295,22 @@ QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescripti
             } */
         }
         break;
+    case Phonon::AudioChannelType:
+    case Phonon::SubtitleType:
+        {
+            ObjectDescriptionHash descriptionHash = XineEngine::objectDescriptions();
+            ObjectDescriptionHash::iterator descIt = descriptionHash.find(type);
+            if(descIt != descriptionHash.end())
+            {
+                ChannelIndexHash indexHash = descIt.value();
+                ChannelIndexHash::iterator indexIt = indexHash.find(index);
+                if(indexIt != indexHash.end() )
+                {
+                    ret = indexIt.value();
+                }
+            }
+        }
+        break;
     }
     return ret;
 }
